import datetime
from zope.component import queryUtility
from zope.interface.verify import verifyObject, verifyClass
from waeup.kofa.interfaces import ICSVExporter
from waeup.kofa.schoolgrades import ResultEntry
from waeup.kofa.students.tests.test_batching import StudentImportExportSetup
from waeup.aaue.utils.utils import CustomKofaUtils
from waeup.aaue.students.export import (
    CustomStudentExporter, CustomStudentPaymentExporter,
    CustomStudentStudyLevelExporter)
from waeup.aaue.testing import FunctionalLayer


class CustomStudentExporterTest(StudentImportExportSetup):

    layer = FunctionalLayer

    def setUp(self):
        super(CustomStudentExporterTest, self).setUp()
        self.setup_for_export()
        result_entry = ResultEntry(
            CustomKofaUtils.EXAM_SUBJECTS_DICT.keys()[0],
            CustomKofaUtils.EXAM_GRADES[0][0]
            )
        self.student.alr_results = [
            result_entry]
        self.student.fst_sit_results = [
            result_entry]
        self.student.scd_sit_results = [
            result_entry]
        return

    def test_ifaces(self):
        # make sure we fullfill interface contracts
        obj = CustomStudentExporter()
        verifyObject(ICSVExporter, obj)
        verifyClass(ICSVExporter, CustomStudentExporter)
        return


    def test_export_all(self):
        # we can really export students
        # set values we can expect in export file
        self.setup_student(self.student)
        exporter = CustomStudentExporter()
        exporter.export_all(self.app, self.outfile)
        result = open(self.outfile, 'rb').read()
        self.assertMatches(
            'adm_code,alr_date,alr_fname,alr_no,alr_results,clr_code,'
            'date_of_birth,def_adm,disabled,email,emp2_end,emp2_position,'
            'emp2_reason,emp2_start,emp_end,emp_position,emp_reason,'
            'emp_start,employer,employer2,father_address,father_name,'
            'father_phone,father_work,firstname,former_matric,fst_sit_date,'
            'fst_sit_fname,fst_sit_no,fst_sit_results,fst_sit_type,hq2_degree,'
            'hq2_disc,hq2_matric_no,hq2_school,hq2_session,hq2_type,'
            'hq_degree,hq_disc,hq_fname,hq_matric_no,hq_school,hq_session,'
            'hq_type,is_staff,lastname,lga,marit_stat,matric_number,'
            'middlename,mother_address,mother_name,mother_phone,'
            'mother_work,nationality,next_kin_address,next_kin_name,'
            'next_kin_phone,next_kin_relation,nysc_lga,nysc_location,'
            'nysc_year,officer_comment,perm_address,personal_updated,'
            'phone,phone_personal,physical_clearance_date,reg_number,'
            'religion,scd_sit_date,scd_sit_fname,scd_sit_no,scd_sit_results,'
            'scd_sit_type,sex,student_id,suspended,suspended_comment,password,'
            'state,history,certcode,is_postgrad,current_level,current_session'
            '\r\n'
            'my adm code,,,,"[(\'printing_craft_practice\', \'A1\')]",'
            'my clr code,1981-02-04#,,,anna@sample.com,,,,,,,,,,,,,,,Anna,,,,,'
            '"[(\'printing_craft_practice\', \'A1\')]",,,,,,,,,,,,,,,,Tester,,,'
            '234,M.,,,,,NG,,,,,,,,,"Studentroad 21\nLagos 123456\n",,'
            '+234-123-12345#,,,123,,,,,'
            '"[(\'printing_craft_practice\', \'A1\')]",,f,A111111,0,,,'
            'created,[u\'2015-10-26 17:45:56 WAT - Record created by system\'],'
            'CERT1,0,200,2012\r\n',
            result
            )
        return

class CustomStudentPaymentExporterTest(StudentImportExportSetup):

    layer = FunctionalLayer

    def setUp(self):
        super(CustomStudentPaymentExporterTest, self).setUp()
        self.setup_for_export()
        return

    def test_ifaces(self):
        # make sure we fullfill interface contracts
        obj = CustomStudentPaymentExporter()
        verifyObject(ICSVExporter, obj)
        verifyClass(ICSVExporter, CustomStudentPaymentExporter)
        return

    def test_export_all(self):
        # we can really export students
        # set values we can expect in export file
        self.setup_student(self.student)
        self.student['payments']['my-payment'].r_company = 'interswatch'
        self.student['payments']['my-payment'].r_card_num = '789'
        exporter = CustomStudentPaymentExporter()
        exporter.export_all(self.app, self.outfile)
        result = open(self.outfile, 'rb').read()
        self.assertMatches(
            'ac,amount_auth,creation_date,gateway_amt,p_category,p_current,'
            'p_id,p_item,p_level,p_session,p_state,payment_date,provider_amt,'
            'r_amount_approved,r_card_num,r_code,r_company,r_desc,'
            'r_pay_reference,thirdparty_amt,student_id,state,'
            'current_session\r\n'
            '666,12.12,2012-04-01 13:12:01#,,schoolfee,1,my-id,p-item,'
            '100,2012,paid,2012-04-01 14:12:01#,,12.12,'
            '789,r-code,interswatch,,,,A111111,created,2012\r\n',
            result
            )
        return

class CustomStudentStudyLevelExporterTest(StudentImportExportSetup):

    layer = FunctionalLayer

    def setUp(self):
        super(CustomStudentStudyLevelExporterTest, self).setUp()
        self.setup_for_export()
        return

    def test_export_all(self):
        # we can really export study levels
        # set values we can expect in export file
        self.setup_student(self.student)
        exporter = CustomStudentStudyLevelExporter()
        exporter.export_all(self.app, self.outfile)
        result = open(self.outfile, 'rb').read()
        self.assertEqual(
            result,
            'gpa,level,level_session,level_verdict,total_credits,'
            'total_credits_s1,total_credits_s2,'
            'validated_by,validation_date,'
            'student_id,number_of_tickets,certcode\r\n'
            '0.0,100,2012,A,100,,,,,A111111,1,CERT1\r\n'
            )
        return

